@@ -1155,9 +1155,10 @@ def write_station_file_gins_from_rinex(rinex_path,station_file_out,
 # |_|  \___|\__,_|\__,_| |_| \_|_|  |_|______/_/    \_\  \_____|\_____/_/    \_\
 #
 
-def read_nmea(file_path , enuout = True ,
-            startdate = dt.datetime(1980,1,1) ,
-            enddate = dt.datetime(2099,1,1) , export_path = ''):
+def read_nmea(file_path , outtype = "FLH" , df_out = True, use_altitude = False,
+              startdate = dt.datetime(1980,1,1) ,
+              enddate = dt.datetime(2099,1,1) ,
+              export_path = ''):
     """ if export_path != '', export a Matlab readable file to this path
         WARNING !!! la coord de ref est codée en dur, à coriger !!!!"""
     T    = []
@@ -1168,16 +1169,31 @@ def read_nmea(file_path , enuout = True ,
     day = 999
     month = 999
     year = 999
+    
+    if use_altitude:
+        field_h_alt = 9
+    else:
+        field_h_alt = 11
+        
+    
+    ## open the file as binary because can be mixed with bin data
+    for l in open(file_path,"rb"):
+        #print(l)
+        try:
+            l = (l).decode('ascii')
+            #print(l)
 
-    for l in open(file_path):
+        except:
+            continue
+        
         if not l[0] == '$':
             continue
-        if 'GPZDA' in l:
+        if 'GPZDA' in l or 'GNZDA' in l:
             f = l.split(',')
             day = int(f[2])
             month = int(f[3])
             year = int(f[4])
-        if 'GPGGA' in l:
+        if 'GPGGA' in l or 'GNGGA' in l:
             f = l.split(',')
             h = int(f[1][0:2])
             m = int(f[1][2:4])
@@ -1199,7 +1215,7 @@ def read_nmea(file_path , enuout = True ,
             T.append(t)
             Lat.append(float(f[2][0:2]) + float(f[2][2:]) / 60.)
             Long.append( EW * (float(f[4][0:3]) + float(f[4][3:]) / 60.))
-            Haut.append(float(f[9]))
+            Haut.append(float(f[field_h_alt]))
             Qual.append(qual)
 
     X,Y,Z = conv.GEO2XYZ(Lat,Long,Haut)
@@ -1221,10 +1237,25 @@ def read_nmea(file_path , enuout = True ,
             datalis = [str(e) for e in datalis] + ['\n']
             outf.write(','.join(datalis)) #)
         outf.close()
-    if enuout:
-        return T,E,N,U,Qual
+        
+    if outtype == 'ENU':
+        OUTTUP = T,E,N,U,Qual
+        colnames = ('T','E','N','U','Qual')
+    elif outtype == 'XYZ':
+        OUTTUP = T,X,Y,Z,Qual
+        colnames = ('T','X','Y','Z','Qual')        
     else:
-        return T,Lat,Long,Haut,Qual
+        OUTTUP = T,Lat,Long,Haut,Qual
+        colnames = ('T','F','L','H','Qual')
+        
+    if df_out:
+        DFout = pd.DataFrame(np.column_stack(OUTTUP))
+        DFout.columns = colnames
+        DFout = DFout.infer_objects()
+        return DFout
+    else:
+        return OUTTUP
+        
 
 #strtd = dt.datetime(2015,06,19,16,00)
 #T,E,N,U,Q = read_nmea(file_path='/home/psakicki/geodesea_nav_final.dat',enuout=1,export_path='/home/psakicki/Documents/geodesea_nav_matrix.dat') #,startdate=strtd)
@@ -1711,8 +1742,7 @@ def read_sinex_versatile(sinex_path_in , id_block,
     DF : Pandas DataFrame
         Returned DataFrame
     """
-
-
+    
     ### remove the + or - if any    
     if id_block in ("+","-"):
         id_block = id_block[1:]
@@ -1734,11 +1764,15 @@ def read_sinex_versatile(sinex_path_in , id_block,
     header_lines = True
     for i_l , l in enumerate(Lines_list):
         if not l[0] in (" ","\n") and header_lines:
+            ## here we store the 1st commented lines i.e. the header
             Lines_list_header.append(l)
-        #if l[0] == " ": ### 1st dataline is excluded if this test is done
-        else:
+        elif l[0] in (" ","\n"):
+            ## here we store the data lines (not commented)
             header_lines = False
             Lines_list_OK.append(l)
+        else:
+            ## here we skip the final commented lines (can happend) 
+            continue
                                 
     if len(Lines_list_header) > 0 and header_line_idx:
         ### define the header
@@ -1751,14 +1785,25 @@ def read_sinex_versatile(sinex_path_in , id_block,
         else: 
             ### Smarter case : we search for the n spaces after the column name
             Fields_size = []
+            
+            
             for fld_head_split in Header_split:
                 if fld_head_split[0] == "*": 
                     ## test to manage a * as 1st character
                     ## which can be wrongly interpreted in the regex
                     fld_head_regex = re.compile("\*" + fld_head_split[1:] + " *") 
                 else:
-                    fld_head_regex = re.compile(fld_head_split + " *") 
+                    fld_head_regex_str = fld_head_split + " *"
+                    ### update 202110: now the brackets must be escaped (normal btw...)
+                    fld_head_regex_str = fld_head_regex_str.replace("[","\[")
+                    fld_head_regex_str = fld_head_regex_str.replace("]","\]")
+                    fld_head_regex_str = fld_head_regex_str.replace("(","\(")
+                    fld_head_regex_str = fld_head_regex_str.replace(")","\)")
+
+                    fld_head_regex = re.compile(fld_head_regex_str) 
+
                 fld_head_space = fld_head_regex.search(header_line)
+
                 Fields_size.append(len(fld_head_space.group()))
                 #print(fld_head_space.group())                
                 
